﻿#Region " Imports "
Imports System.Data
Imports Microsoft.VisualBasic
#End Region

''' <summary>
''' Data section building dummy data for testing purposes.
''' </summary>
''' <remarks></remarks>
Public Class DataLayer
#Region " Constants "
    Public Const HIDDEN_FIELDNAME_PREFIX As String = "Hidden "
    Private Const RECORD_COUNT As Integer = 100
    Private Const DETAILS_STRING As String = "Etsi difficiile esse videtur credere quicquam" & _
                                             "in rebus solido reperiri corpore posse." & _
                                             "transit enim fulmen caeli per saepta domorum," & _
                                             "clamor ut ad voces; flamen candescit in igni" & _
                                             "dissiliuntque ferre ferventi saxa vapore." & _
                                             "tum labefactatus rigor auri solvitur aestu;" & _
                                             "tum glacies aeris flamma devicta liquescit;" & _
                                             "permanat calor argentum penetraleque frigus" & _
                                             "quando utrumque manu retinentes pocula rite" & _
                                             "sensimus infuso lympharum rore superne."
#End Region

#Region " Classes "
    ''' <summary>
    ''' Fieldname object.
    ''' </summary>
    ''' <remarks></remarks>
    Public Class FieldName
        Public Const ALERT As String = "ALERT"
        Public Const ID As String = "ID"
        Public Const KEY As String = "KEY"
        Public Const ITEM As String = "ITEM"
        Public Const DATA As String = "DATA"
        Public Const DATA1 As String = "DATA1"
        Public Const RELATION As String = "RELATION"
    End Class
#End Region

#Region " Methods "
    ''' <summary>
    ''' Build a dummy data table.
    ''' </summary>
    ''' <param name="name"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function BuildTable(name As String) As DataTable

        Dim table As System.Data.DataTable = New DataTable(name)
        Dim column As DataColumn
        Dim row As DataRow

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.Int32")
        column.ColumnName = FieldName.ID
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.ALERT
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.Int32")
        column.ColumnName = FieldName.KEY
        column.Caption = HIDDEN_FIELDNAME_PREFIX & FieldName.KEY
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.ITEM
        column.AutoIncrement = False
        column.Caption = FieldName.ITEM
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.DATA
        column.AutoIncrement = False
        column.Caption = FieldName.DATA
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.DATA1
        column.AutoIncrement = False
        column.Caption = FieldName.DATA1
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        For i As Integer = 1 To CInt(RECORD_COUNT)
            row = table.NewRow()
            row(FieldName.ID) = i
            row(FieldName.ALERT) = "&amp;"
            row(FieldName.KEY) = i - 1
            row(FieldName.ITEM) = FieldName.ITEM & " " & i
            row(FieldName.DATA) = FieldName.DATA & " - &amp;" & DETAILS_STRING & " [" & i & "]."
            row(FieldName.DATA1) = FieldName.DATA1 & " " & i
            table.Rows.Add(row)
        Next
        Return table

    End Function

    ''' <summary>
    ''' Creates a Child data table.
    ''' </summary>
    ''' <param name="name"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function BuildChildTable(name As String) As DataTable
        Dim table As System.Data.DataTable = New DataTable(name)
        Dim column As DataColumn
        Dim row As DataRow

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.Int32")
        column.ColumnName = FieldName.ID
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.ALERT
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.Int32")
        column.ColumnName = FieldName.KEY
        column.Caption = HIDDEN_FIELDNAME_PREFIX & FieldName.KEY
        column.[ReadOnly] = True
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.ITEM
        column.AutoIncrement = False
        column.Caption = FieldName.ITEM
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.DATA
        column.AutoIncrement = False
        column.Caption = FieldName.DATA
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.String")
        column.ColumnName = FieldName.DATA1
        column.AutoIncrement = False
        column.Caption = FieldName.DATA1
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        column = New DataColumn()
        column.DataType = System.Type.[GetType]("System.Int32")
        column.ColumnName = FieldName.RELATION
        column.AutoIncrement = False
        column.Caption = FieldName.RELATION
        column.[ReadOnly] = False
        column.Unique = False
        table.Columns.Add(column)

        Dim PrimaryKeyColumns As DataColumn() = New DataColumn(0) {}
        PrimaryKeyColumns(0) = table.Columns(FieldName.ID)
        table.PrimaryKey = PrimaryKeyColumns

        For i As Integer = 0 To 100
            row = table.NewRow()
            row(FieldName.ID) = i
            row(FieldName.ALERT) = "&amp;"
            row(FieldName.ITEM) = FieldName.ITEM & " " & i
            row(FieldName.DATA) = FieldName.DATA & " " & DETAILS_STRING & " [" & i & "]"
            row(FieldName.DATA1) = FieldName.DATA & " " & i

            If (i < 10) AndAlso (i >= 0) Then
                row(FieldName.RELATION) = 0
            ElseIf (i < 20) AndAlso (i >= 10) Then
                row(FieldName.RELATION) = 1
            ElseIf (i < 30) AndAlso (i >= 20) Then
                row(FieldName.RELATION) = 2
            ElseIf (i < 40) AndAlso (i >= 30) Then
                row(FieldName.RELATION) = 3
            ElseIf (i < 50) AndAlso (i >= 40) Then
            End If
            table.Rows.Add(row)
        Next
        Return table

    End Function
#End Region
End Class
