﻿#Region " Imports "
Imports DataLayer

Imports System.Data

Imports Infragistics.Web.UI.GridControls
Imports Infragistics.Web.UI.Framework
Imports Infragistics.Web.UI
#End Region

Partial Class _Default
    Inherits Page

#Region " Declarations "
    Private Const DEF_PAGE_SIZE As Integer = 10
    Private Const DEF_COLUMN_WIDTH As Integer = 80

    Private _testDataSet As DataSet
    Private _groupByString As String
    Private _groupedColumnSettings As GridGroupedColumnCollection
#End Region

#Region " Classes "
    Protected Class CSSClass
        Public Const GridRow As String = "gridrow"
        Public Const SelectedGridRow As String = "gridselectedrow"
    End Class
#End Region

#Region " Properties "
    Public Property TestData() As DataTable
        Get
            Dim oDataTable As Object = ViewState(ViewStateVariable.TestDataTable)
            If IsNothing(oDataTable) Then
                Return New DataTable()
            End If
            Return DirectCast(oDataTable, DataTable)
        End Get
        Set(value As DataTable)
            ViewState(ViewStateVariable.TestDataTable) = value
        End Set
    End Property

    Public ReadOnly Property TestDataSet() As DataSet
        Get
            If IsNothing(Me.TestData.DataSet) Then
                Dim dsData As New DataSet()
                dsData.Tables.Add(Me.TestData)
            End If
            Return Me.TestData.DataSet
        End Get
    End Property

    Public Property GroupByString() As String
        Get
            Return _groupByString
        End Get
        Set(value As String)
            Me._groupByString = value
        End Set
    End Property

    ''' <summary>
    ''' Gets a value indicating whether this instance's <see cref="GroupedColumnSettings"/> property has been initialised.
    ''' </summary>
    ''' <value><c>true</c> if this instance's <see cref="GroupedColumnSettings"/> property has been initialised; otherwise, <c>false</c>.</value>
    Public ReadOnly Property HasGroupedColumnSettings() As Boolean
        Get
            Return (Me._groupedColumnSettings IsNot Nothing)
        End Get
    End Property

    ''' <summary>
    ''' Gets the grouped column settings.
    ''' </summary>
    ''' <value>The grouped column settings.</value>
    ''' <remarks>Returns an empty <see cref="GridGroupedColumnCollection" /> if <c>GroupedColumnSettings</c> property hasn't been initialised.
    ''' Inspect the <see cref="HasGroupedColumnSettings" /> property to determine if the <c>GroupedColumnSettings</c> property 
    ''' has been initialised.
    ''' </remarks>
    Public ReadOnly Property GroupedColumnSettings() As GridGroupedColumnCollection
        Get
            If Not (Me.HasGroupedColumnSettings()) Then
                Me._groupedColumnSettings = New GridGroupedColumnCollection
            End If
            Return Me._groupedColumnSettings
        End Get
    End Property
#End Region

#Region " Page Events "
    ''' <summary>
    ''' Occurs when the server control is loaded into the System.Web.UI.Page object.
    ''' </summary>
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        If Not (Me.IsPostBack) Then
            Me.TestData = BuildChildTable("Parent")
            Me.FillGridPage()
        End If

    End Sub
#End Region

#Region " Events "
    ''' <summary>
    ''' Occurs when a column is moved by a user.
    ''' </summary>
    Protected Sub whdg_ColumnMoved(sender As Object, e As ColumnMovingEventArgs) Handles whdg.ColumnMoved
        If IsNothing(e.Column) Then
            Return
        End If

        Dim movedColumn As ControlDataField = e.Column
        Dim dataColumn As DataColumn = Me.TestData.Columns(e.Column.Key)
        Dim oldVisiblieIndex As Integer = e.PreviousVisibleIndex
        Dim newVisibleIndex As Integer = movedColumn.VisibleIndex
        If newVisibleIndex <> oldVisiblieIndex Then
            dataColumn.SetOrdinal(newVisibleIndex)
        End If

        Me.FillGridPage()

    End Sub

    ''' <summary>
    ''' Occurs when a column is resized by a user.
    ''' </summary>
    Protected Sub whdg_ColumnResized(sender As Object, e As ColumnResizingEventArgs) Handles whdg.ColumnResized
        If IsNothing(e.Column) Then
            Return
        End If

        Me.FillGridPage()
    End Sub

    ''' <summary>
    ''' Occurs when a column is sorted by a user.
    ''' </summary>
    Protected Sub whdg_ColumnSorted(sender As Object, e As SortingEventArgs) Handles whdg.ColumnSorted
        If IsNothing(e.Column) Then
            Return
        End If

        Me.FillGridPage()
    End Sub

    ''' <summary>
    ''' The InitializeRow event is raised when the grid binds to the records in the data source.
    ''' </summary>
    Private Sub whdg_InitializeRow(sender As Object, e As RowEventArgs) Handles whdg.InitializeRow
        Initialize_HierarchicalGridRow(e)
    End Sub

    ''' <summary>
    ''' Occurs when a page has changed by a user.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Sub whdg_PageIndexChanged(sender As Object, e As Infragistics.Web.UI.GridControls.PagingEventArgs) Handles whdg.PageIndexChanged

        Dim newPageIndex As Integer = Me.whdg.GridView.Behaviors.Paging.PageIndex

        Me.FillGridPage()

        Me.whdg.GridView.Behaviors.Paging.PageIndex = newPageIndex

    End Sub

    ''' <summary>
    ''' Occurs when the selection from the list control changes between posts to the server.
    ''' </summary>
    Protected Sub ddlGroupBy_SelectedIndexChanged(sender As Object, e As System.EventArgs) Handles ddlGroupBy.SelectedIndexChanged

        Dim groupbyColumn As String = Me.ddlGroupBy.SelectedItem.Value

         Select groupbyColumn
            Case FieldName.ID, FieldName.DATA
                Me.whdg.GridView.ClearDataSource()
                Me.TestData = BuildTable("Parent")
                Me.FillGridPage()
            Case "None"
                Me.whdg.GridView.ClearDataSource()
                Me.TestData = BuildChildTable("Parent")
                Me.FillGridPage()
        End Select
    End Sub
#End Region

#Region " Methods "
    ''' <summary>
    ''' Binds data rows to a <see cref="WebHierarchicalDataGrid">WebHierarchicalDataGrid</see>.
    ''' </summary>
    Protected Overridable Sub FillGridPage()

        Dim sortDirection As String = " " & Tsql.SortIndicator.Ascending.ToString
        Dim groupbyColumn As String = Me.ddlGroupBy.SelectedItem.Value
        Me.whdg.Key = "Grid Grouping Test"

        Select Case groupbyColumn
            Case FieldName.ID
                Me.GroupByString = GetSqlGroupBy(FieldName.ID & sortDirection)

                Initialize_GridColumns(Me.whdg, Me.TestDataSet)
                InitializeHierarchicalGridGrouping(Me.whdg)

                Me.whdg.DataKeyFields = FieldName.ID & "," & FieldName.KEY
                Me.whdg.GridView.ClearDataSource()
                Me.whdg.DataSource = Me.TestDataSet
            Case FieldName.DATA
                Me.GroupByString = GetSqlGroupBy(FieldName.DATA & sortDirection)

                Initialize_GridColumns(Me.whdg, Me.TestDataSet)
                InitializeHierarchicalGridGrouping(Me.whdg)

                Me.whdg.DataKeyFields = FieldName.DATA & "," & FieldName.KEY
                Me.whdg.GridView.ClearDataSource()
                Me.whdg.DataSource = Me.TestDataSet
            Case "None"
                Me.GroupByString = String.Empty

                Initialize_GridColumns(Me.whdg, Me.TestDataSet)
                InitializeHierarchicalGridGrouping(Me.whdg)

                Me.whdg.DataKeyFields = FieldName.ID
                Me.whdg.GridView.ClearDataSource()
                Me.whdg.DataSource = Me.TestDataSet
        End Select

        Initialize_GridLayout(Me.whdg)

        'DataBind() ' This fixes the problem ---> not sure if this fix can be used in the app due to the diary tab complexity.

    End Sub

    ''' <summary>
    ''' Initialize any custom grid properties.
    ''' </summary>
    Private Sub Initialize_GridLayout(ByRef grid As WebHierarchicalDataGrid)
        grid.GridView.Behaviors.Paging.PageSize = CInt(DEF_PAGE_SIZE)
    End Sub

    ''' <summary>
    ''' Initialize the grid columns collection.
    ''' </summary>
    Private Sub Initialize_GridColumns(ByRef grid As WebHierarchicalDataGrid, ByVal dsData As DataSet)
        If grid.GridView.Columns.Count > 0 Then
            grid.GridView.Columns.Clear()
        End If

        Dim gridColumns As DataColumnCollection
        gridColumns = dsData.Tables(0).Columns
        For i As Integer = 0 To gridColumns.Count - 1
            Dim column As DataColumn = gridColumns.Item(i)
            Initialize_HierarchicalGridColumn(grid.GridView, column)
            Format_HierarchicalGridColumn(grid.GridView, _
                                          column, _
                                          column.Caption.Contains(HIDDEN_FIELDNAME_PREFIX.Trim))

        Next

    End Sub

    ''' <summary>
    ''' Initialize a grid column.
    ''' </summary>
    Private Sub Initialize_HierarchicalGridColumn(ByRef band As IBand, ByVal oDataColumn As DataColumn)
        Dim column As New BoundDataField()
        column.Key = oDataColumn.ColumnName
        column.DataType = oDataColumn.DataType.ToString
        column.DataFieldName = oDataColumn.ColumnName
        band.Columns.Add(column)
    End Sub

    ''' <summary>
    ''' Formats a grid column.
    ''' </summary>
    Private Sub Format_HierarchicalGridColumn(ByRef band As IBand, ByVal oDataColumn As DataColumn, ByVal hidden As Boolean)

        band.Columns(oDataColumn.ColumnName).Header.Text = oDataColumn.ColumnName
        band.Columns(oDataColumn.ColumnName).Header.Tooltip = oDataColumn.ColumnName
        band.Columns(oDataColumn.ColumnName).Hidden = hidden
        band.Columns(oDataColumn.ColumnName).Width = Unit.Pixel(DEF_COLUMN_WIDTH)

    End Sub

    ''' <summary>
    ''' Format the grid row fields.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Initialize_HierarchicalGridRow(ByRef e As Infragistics.Web.UI.GridControls.RowEventArgs)
        Dim sClass As String = ""

        e.Row.CssClass = e.Row.CssClass & " " & CSSClass.GridRow 'sClass
        If e.Row.Items.Count > 0 Then 'Make sure there are columns to process
            For i As Integer = 0 To e.Row.Items.Count - 1
                With e.Row.Items.Item(i) 'Iterate through row cells and apply CssClasses
                    Select Case .Column.Header.FieldKey
                        Case FieldName.ID
                            .Text = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            .Tooltip = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(
                                create_FontCssClass("red"), "tbody  tr  td.{0}", False)
                            .CssClass = sClass
                        Case FieldName.ALERT
                            .Text = String.Empty
                            .Tooltip = HttpUtility.HtmlEncode("Alert")
                            .CssClass = "gridcelloverdue"
                        Case FieldName.ITEM
                            .Text = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            .Tooltip = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(
                                create_FontCssClass("black"), "tbody  tr  td.{0}", False)
                            .CssClass = sClass
                        Case FieldName.DATA
                            .Text = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            .Tooltip = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            sClass = AppStyling.AppStylingManager.Settings.CssRegistry.Add(
                                create_FontCssClass("black"), "tbody  tr  td.{0}", False)
                            .CssClass = sClass
                        Case FieldName.DATA1
                            .Text = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                            .Tooltip = HttpUtility.HtmlEncode(.Value.ToString.Trim)
                    End Select
                End With
            Next
        End If
    End Sub

    ''' <summary>
    ''' Initialize the webhierarchicaldatagrid column grouping settings.
    ''' </summary>
    Private Sub InitializeHierarchicalGridGrouping(ByVal grid As WebHierarchicalDataGrid)
        Dim groupbyColumn As String = Me.ddlGroupBy.SelectedItem.Text
        Dim newGroupCollection As New GridGroupedColumnCollection
        grid.GroupingSettings.GroupedColumns.Clear()

        Dim groupCriteria As New GridGroupedColumnCollection(Me.GroupByString)
        For Each column As GridGroupedColumn In groupCriteria
            If Not (Me.GroupedColumnSettings.Contains(column)) Then
                newGroupCollection.Add(column)
            End If
        Next

        For Each column As GridGroupedColumn In newGroupCollection
            grid.GroupingSettings.GroupedColumns.Add(column.Name, column.SortDirection)
        Next

        Me.GroupedColumnSettings.Clear()
        Me.GroupedColumnSettings.AddRange(newGroupCollection)
        Me.GroupByString = Me.GroupedColumnSettings.ToSql

    End Sub

    ''' <summary>
    ''' Clears the webhierarchicaldatagrid column grouping settings.
    ''' </summary>
    Private Sub ClearHierarchicalGridGrouping(ByVal grid As WebHierarchicalDataGrid)
        Dim groupbyColumn As String = Me.ddlGroupBy.SelectedItem.Text
        Dim newGroupCollection As New GridGroupedColumnCollection
        grid.GroupingSettings.GroupedColumns.Clear()

        For Each column As GridGroupedColumn In newGroupCollection
            grid.GroupingSettings.GroupedColumns.Add(column.Name, column.SortDirection)
        Next

        Me.GroupedColumnSettings.Clear()
        Me.GroupedColumnSettings.AddRange(newGroupCollection)

    End Sub

    ''' <summary>
    ''' Returns a sql groupby criteria string.
    ''' </summary>
    Public Shared Function GetSqlGroupBy(strGroupBy As String) As String
        Dim groupCriteria As New GroupByCollection
        groupCriteria.FromSql(strGroupBy)
        strGroupBy = groupCriteria.ToSql()
        Return strGroupBy
    End Function

    ''' <summary>
    ''' Creates a row css class string.
    ''' </summary>
    ''' <param name="strRowHeight"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function create_RowCssClass(ByVal strRowHeight As String) As String
        Dim sClass As String = String.Format("height:{0};", strRowHeight)
        Return sClass
    End Function

    ''' <summary>
    ''' Sets the row css class attributes.
    ''' </summary>
    ''' <param name="strColour"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function create_FontCssClass(ByVal strColour As String) As String
        Dim sClass As String = String.Format("background-position:center; " & _
                                             "background-repeat:no-repeat; " & _
                                             "border-bottom:solid 1px #D8D8D8; " & _
                                             "overflow: hidden; " & _
                                             "text-overflow:ellipsis; " & _
                                             "white-space:nowrap; " & _
                                             "color:{0};", strColour)
        Return sClass
    End Function

    ''' <summary>
    ''' Returns the client-side page header meta tag.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Protected Overridable Function GetClientIncludes() As String
        Return String.Format( _
            "<script type=""text/javascript"" src=""{0}""></script>", "common.js")
    End Function

    ''' <summary>
    ''' Returns a boolean indicating if the column key is found in the grid columns collection.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function IsColumnKeyFound(ByVal columnKey As String) As Boolean

        Dim field As ControlDataField = Me.whdg.GridView.Columns.FromKey(columnKey)

        If Not (IsNothing(field)) Then
            Return True
        End If
        Return False
    End Function

    ''' <summary>
    ''' Returns a Boolean value checking if a control exists in the Page Header section.
    ''' </summary>
    Public Function IsStyleSheetIncludeRegistered(key As String) As Boolean
        For Each control As Control In Page.Header.Controls
            If (control.ID = key) Then
                Return True
            End If
        Next
        Return False
    End Function

    ''' <summary>
    ''' Registers the style sheet include in the body section.
    ''' </summary>
    ''' <param name="key">The control ID to make sure there aren't two
    ''' registered under the same ID.</param>
    ''' <param name="src">The file name.</param>
    Public Sub RegisterStyleSheetInclude(key As String, src As String)

        Dim styleSheetInclude As New HtmlLink _
            With {
                .ID = key,
                .Href = src
            }
        styleSheetInclude.Attributes.Add("type", "text/css")
        styleSheetInclude.Attributes.Add("rel", "stylesheet")
        Page.Header.Controls.Add(styleSheetInclude)
    End Sub
#End Region

#Region " Overrides "
    ''' <summary>
    ''' Raises the System.Web.UI.Control.PreRender event.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Page_PreRender(sender As Object, e As System.EventArgs) Handles MyBase.PreRender
        ' Register client script
        Dim strScriptKey As String = Me.GetType().ToString()

        If Not (ClientScript.IsClientScriptBlockRegistered(strScriptKey)) Then
            ClientScript.RegisterClientScriptBlock(GetType(Page), strScriptKey, Me.GetClientIncludes())
        End If

        ' Register start-up script
        If Not (ClientScript.IsStartupScriptRegistered(strScriptKey)) Then
            Dim sbScript As New StringBuilder("<script type=""text/javascript"">")
            sbScript.Append(vbNewLine)
            sbScript.Append("<!--")
            sbScript.Append(vbNewLine)
            sbScript.Append("//-->")
            sbScript.Append(vbNewLine)
            sbScript.Append("</script>")
            ClientScript.RegisterStartupScript(GetType(Page), strScriptKey, sbScript.ToString)
        End If

        
        If Not (Me.IsStyleSheetIncludeRegistered("jc_Styles")) Then
            Me.RegisterStyleSheetInclude("jc_Styles", ResolveClientUrl("Styles.css"))
        End If

    End Sub
#End Region
End Class
